"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Tries to convert dates in string-form to an actual Date object.
var dateModifier = function () { return function (input) {
    return input instanceof Array
        ? input.map(function (a) { return dateModifier()(a); })
        : isNaN(+new Date(input))
            ? input
            : new Date(input);
}; };
var aliasKeys = [
    {
        from: ['domain name', 'domain'],
        to: 'domain'
    },
    {
        from: ['expiry date', 'registry expiry date'],
        to: 'expiration',
        modifier: dateModifier()
    },
    {
        from: ['created', 'creation date'],
        to: 'created',
        modifier: dateModifier()
    },
    {
        from: ['updated date', 'last-update'],
        to: 'updated',
        modifier: dateModifier()
    },
    {
        from: ['status', 'domain status'],
        to: 'status'
    },
    {
        from: ['registrar'],
        to: 'registrar'
    },
    {
        from: ['registrar iana id'],
        to: 'registrarIANAId'
    },
    {
        from: ['registrar abuse contact email'],
        to: 'registrarAbuseContactEmail'
    },
    {
        from: ['registrar abuse contact phone'],
        to: 'registrarAbuseContactPhone'
    },
    {
        from: ['name server', 'nserver'],
        to: 'nameServer'
    },
    {
        from: ['dnssec'],
        to: 'DNSSEC'
    }
];
var nonExistentSignatures = [
    'NOT FOUND',
    'No Data Found',
    'No match for',
    'This domain name has not been registered.',
    'Domain not found',
    'The queried object does not exist',
    'El dominio no se encuentra registrado',
    'is available for registration',
    'is free',
    'Invalid domain name',
    'Domain Not Found',
    'Object does not exist',
    'domain name not known',
    'No matching record.',
    'No information was found matching that query.',
    'not found...',
    'syntax error in specified domain name',
    'NO MATCH',
    'No entries found',
    'not found',
    'The domain has not been registered'
];
/**
 * Filters out commeted lines and converts every line to a
 * JS object key–value pair with multiple values made into an array.
 *
 * @param {[key: string]: string | string[]} data – the raw WHOIS response data
 */
exports.parseWhois = function (data) {
    return data
        .split('\n')
        .filter(function (line) { return line.length > 0 && !line.startsWith('%'); })
        .map(function (line) { return line.split(/:\s+/); })
        .map(function (line) { return ({
        key: line[0].trim(),
        value: line
            .slice(1)
            .join(': ') // if there happens to be `: ` somewhere in the property value, this will restore them (not accounting for extra spaces)
            .trim()
    }); })
        .reduce(function (previous, _a) {
        var key = _a.key, value = _a.value;
        return (__assign({}, previous, (_b = {},
            _b[key] = previous[key] // if there is already a property using the same key, create an array containing all values.
                ? (previous[key] instanceof Array
                    ? previous[key]
                    : [previous[key]]).concat([
                    value
                ]) : value,
            _b)));
        var _b;
    }, {});
};
/**
 * Tries to find an alias and a modifier by the key.
 * Provies a default modifier function that returns the argument if no modifier found.
 *
 * @param {string} key – the key that'll be used to find the alias
 */
var findAliasByKey = function (key) { return (__assign({ to: key, modifier: function (a) { return a; } }, (aliasKeys.find(function (_a) {
    var from = _a.from;
    return from.includes(key.toLowerCase());
}) || {}))); };
/**
 * Parses given raw WHOIS data to JS object form
 *
 * @param {string} data – raw WHOIS data to parse from
 */
exports.default = (function (data) { return (__assign({ exists: nonExistentSignatures.findIndex(function (sign) {
        return new RegExp(sign, 'i').test(data);
    }) === -1 }, Object.entries(exports.parseWhois(data))
    .map(function (_a) {
    var key = _a[0], value = _a[1];
    return ({ value: value, alias: findAliasByKey(key) });
})
    .map(function (_a) {
    var value = _a.value, alias = _a.alias;
    return [alias.to, alias.modifier(value)];
})
    .reduce(function (p, n) {
    return (__assign({}, p, (_a = {}, _a[n[0]] = n[1], _a)));
    var _a;
}, {}))); });
